#!/usr/bin/python

#
# KconfigSize tool---bind memory/size result and conditions
# Copyright (C) 2005-2006 NEC-AS/NEC. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GPLv2, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Created by Mingwei Wang <wangmw@sh.necas.nec.com.cn>
# 2006-02-20 Modified by Mingwei Wang <wangmw@sh.necas.nec.com.cn>
#

import os, sys, string, re
import commands

MAJOR_VERSION = 0
MINOR_VERSION = 1

# keep configuration file in /etc

configs = {}


def usage(outfile):
    outfile.write("""Usage: %s -t <type> [-d <dir>]|[-i <file> -c <file> -r <file>] <output file>

Meta-options:
-t, --type            the type of binder, \"mem\" or \"size\"

-d, --dir             directory of files
Otherwise, the following three options must be given:
-i, --index           index file
-c, --condition       condition file
-r, --result          result file

-h, --help            Display this help then exit.
-v, --version         Output version information then exit.
""" % sys.argv[0])

def print_error(message):
	sys.stderr.write("Error: "+message+"\n")
	sys.stderr.flush()

def error_out(message, rcode):
	print_error(message)
	sys.exit(rcode)

def binder(i_file, c_file, r_file, o_file):

    try:
	    fi = open(i_file)
    except:
	    error_out("Cannot open index file %s" % i_file, 3)
    try:
	    fc = open(c_file)
    except:
	    error_out("Cannot open condition file %s" % c_file, 3)
    try:
	    fr = open(r_file)
    except:
	    error_out("Cannot open result file %s" % r_file, 3)
    try:
	    fo = open(o_file,'w')
    except:
	    error_out("Cannot open output file %s" % o_file, 3)

    if type == 'size':
	fo.write('---size result\n')
    if type == 'mem':
	fo.write('---memory result\n')
    while True:
	val = []
	all_val = []

	linei = fi.readline()
	linei = linei.strip()
	liner = fr.readline()
	liner = liner.strip()
	if not linei:
	    break
	fo.write(linei)

	val = liner.split(",")
	del val[0]
	for tmp in val:
	    fo.write(',')
	    fo.write(tmp)
	fo.write('\n')

    fo.write('---conditions\n')
    for linec in fc.readlines():
	fo.write(linec)
    fo.close()


def main(argv=None):
    import getopt
    global type

    type=""
    files_dir=""
    index_file=""
    condition_file=""
    result_file=""
    output_file=""
 
    if argv is None:
        argv = sys.argv
    try:
        opts, prog_argv = getopt.getopt(argv[1:], "t:d:i:c:r:hv",
                                        ["help", "version",])

    except getopt.error, msg:
        sys.stderr.write("%s: %s\n" % (sys.argv[0], msg))
        sys.stderr.write("Try `%s --help' for more information\n"
                         % sys.argv[0])
        sys.exit(1)

    for opt, val in opts:
        if opt == "-h" or opt == "--help":
            usage(sys.stdout)
            sys.exit(0)

        if opt == "-v" or opt == "--version":
            sys.stdout.write("BINDMEM version %d.%d\n" % (MAJOR_VERSION, MINOR_VERSION))
            sys.exit(0)

        if opt == "-t" or opt == "--type":
            type = val
            continue

        if opt == "-d" or opt == "--dir":
            files_dir = val
            continue

        if opt == "-i" or opt == "--index":
            index_file = val
            continue

        if opt == "-c" or opt == "--condition":
            condition_file = val
            continue

        if opt == "-r" or opt == "--result":
            result_file = val
            continue

        assert 0, "Should never get here"

    if len(prog_argv) == 1:
        output_file = prog_argv[0]
    else:
        sys.stderr.write("output file?\n")
        sys.stderr.write("Try `%s --help' for more information\n"
                         % sys.argv[0])
        sys.exit(1)

    if index_file and condition_file and result_file:
	print ""
    elif (not index_file) and (not condition_file) and (not result_file):
	# everything is ready?
	if not files_dir:
	    files_dir = "./"
	else:
	    files_dir = files_dir +"/"

	index_file=files_dir+"index"
	condition_file=files_dir+"conditions"

	if type == "mem":
	    result_file=files_dir+"mem_result"
	elif type == "size":
	    result_file=files_dir+"size_result"
	else:
	    sys.stderr.write("Try `%s --help' for more information\n"
			 % sys.argv[0])
	    sys.exit(1)
    else:
	sys.stderr.write("Try `%s --help' for more information\n"
		     % sys.argv[0])
	sys.exit(1)

    binder(index_file, condition_file, result_file, output_file)

if __name__=="__main__":
	main()
