#!/usr/bin/python

#
# KconfigSize tool---compare two indiv out files which are the same type(size or mem)
# Copyright (C) 2005-2006 NEC-AS/NEC. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GPLv2, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Created by Mingwei Wang <wangmw@sh.necas.nec.com.cn>
# 2006-02-20 Modified by Mingwei Wang <wangmw@sh.necas.nec.com.cn>
#

import os, sys, string, re
import commands

MAJOR_VERSION = 0
MINOR_VERSION = 1

# keep configuration file in /etc

def usage(outfile):
    outfile.write("""Usage: %s -c <file> [-k <file>] -o <output file> <file1> <file2>

Meta-options:
-c, --condition       condition file
-k, --key             key file
-o, --output          output file

-h, --help            Display this help then exit.
-v, --version         Output version information then exit.
""" % sys.argv[0])

def print_error(message):
	sys.stderr.write("Error: "+message+"\n")
	sys.stderr.flush()

def error_out(message, rcode):
	print_error(message)
	sys.exit(rcode)

# condition file syntax:
# ------------------------
# empty lines and lines starting with # are ignored
# single-line attributes are:
# name=value

def read_condition(c_file):
    conditions = {}

    try:
	fl = open(c_file)
    except:
	error_out("Cannot open condition file %s" % c_file, 3)

    line_no = 0
    for line in fl.readlines():
	line_no += 1
	if line.startswith("#"):
	    continue

	# OK, it's not a condition, comment.
	# check if it's empty
	if not line.strip():
	    continue

	# line better have an equals in it
	# (single line name=value)
	if line.find("=")==-1:
	    print_error("Syntax error in condition file %s: Expected '=' at line %d:\n%s" % (info_file, line_no, line))
	    continue
	
	(name, value) = line.split('=', 1)
	name = name.strip()
	value = value.strip()
	conditions[name] = value

#    keys = conditions.keys()
#    keys.sort()
#    for i in keys:
#	print"%s: %s" % (i,conditions[i])

    return conditions

def find_keys(k_file, file1, file2):
    keys = []

    try:
	    f1 = open(file1)
    except:
	    error_out("Cannot open file %s" % file1, 3)
    try:
	    f2 = open(file2)
    except:
	    error_out("Cannot open file %s" % file2, 3)

    if k_file:
	try:
		fk = open(k_file)
	except:
		error_out("Cannot open key file %s" % k_file, 3)

	for linek in fk.readlines():
	    if linek.startswith("#"):
		continue

	    # OK, it's not a config, comment.
	    # check if it's empty
	    if not linek.strip():
		continue
	    linek = linek.strip()

	    keys.append(linek)
    else:
	val = []
	line1 = f1.readline()
	line1 = f1.readline()
	for line1 in f1.readlines():
	    if line1.startswith("---"):
		break
	    line1 = line1.strip()
	    val = line1.split(",")
	    if val[0] not in keys:
		keys.append(val[0])
	line2 = f2.readline()
	line2 = f2.readline()
	for line2 in f2.readlines():
	    if line2.startswith("---"):
		break
	    line2 = line2.strip()
	    val = line2.split(",")
	    if val[0] not in keys:
		keys.append(val[0])

    return keys

def calc_len(file1, file2):
    try:
	    f1 = open(file1)
    except:
	    error_out("Cannot open file %s" % file1, 3)
    try:
	    f2 = open(file2)
    except:
	    error_out("Cannot open file %s" % file2, 3)

    line1 = f1.readline()
    line1 = f1.readline()
    line1 = f1.readline()
    line1 = line1.strip()
    val = line1.split(",")
    typelen = len(val) - 1

    line2 = f2.readline()
    line2 = f2.readline()
    line2 = f2.readline()
    line2 = line2.strip()
    val = line2.split(",")
    tmplen = len(val) - 1

    if typelen != tmplen:
	error_out("the type of two files %s,%s is not same" % (file1,file2), 3)

    return typelen

def gen_title(file1, file2):
    title = []
    vals = []

    try:
	    f1 = open(file1)
    except:
	    error_out("Cannot open file %s" % file1, 3)
    try:
	    f2 = open(file2)
    except:
	    error_out("Cannot open file %s" % file2, 3)

    line1 = f1.readline()
    line1 = f1.readline()
    line1 = line1.strip()
    vals = line1.split(",")
    title = [vals[0]]
    del vals[0]
    for val in vals:
	title.append(val+'1')

    line2 = f2.readline()
    line2 = f2.readline()
    line2 = line2.strip()
    vals = line2.split(",")
    del vals[0]
    for val in vals:
	title.append(val+'2')

    return title

def read_allvals(keys, typelen, file1, file2):
    vals = []
    allvals = {}

    try:
	    f1 = open(file1)
    except:
	    error_out("Cannot open file %s" % file1, 3)
    try:
	    f2 = open(file2)
    except:
	    error_out("Cannot open file %s" % file2, 3)

    for key in keys:
	#print "key: %s" % key
	f1.close()
	f1 = open(file1)
	for line1 in f1.readlines():
	    line1 = line1.strip()
	    vals = line1.split(",")
	    if len(vals) > 1:
		if vals[0] != key:
		    continue
		else:
		    allvals[key] = [vals[0]]
		    del vals[0]
		    for tmp in vals:
			allvals[key].append(tmp)
		    i=0
		    while (i < typelen):
			allvals[key].append('')
			i=i+1

	f2.close()
	f2 = open(file2)
	for line2 in f2.readlines():
	    line2 = line2.strip()
	    vals = line2.split(",")
	    if len(vals) > 1:
		#print "vals: %s" % vals[0]
		if vals[0] != key:
		    continue
		else:
		    if allvals.has_key(key):
			del vals[0]
			i=1
			for tmp in vals:
			    allvals[key][typelen+i] = tmp
			    i=i+1
		    else:
			allvals[key] = [vals[0]]
			i=0
			while (i < typelen):
			    allvals[key].append('')
			    i=i+1
			del vals[0]
			for tmp in vals:
			    allvals[key].append(tmp)

    f1.close()
    f2.close()
    return allvals

def getCmpFunc(typelen,n) :
    def cmpfn(t1, t2) :
	return ( int(t2[typelen+n])-int(t2[n]) )-( int(t1[typelen+n])-int(t1[n]) )
    return cmpfn

def data_sort(allvals, typelen, num):
    sortdata = []

    okeys = allvals.keys()
    for key in okeys:
	linevals = []
	for val in allvals[key]:
	    if val == '':
		linevals.append('0')
	    else:
		linevals.append(val)
	sortdata.append(linevals)

    sortdata.sort(getCmpFunc(typelen,num))

    return sortdata

def find_num(file1, file2, conditions):
    try:
	    f1 = open(file1)
    except:
	    error_out("Cannot open file %s" % file1, 3)
    try:
	    f2 = open(file2)
    except:
	    error_out("Cannot open file %s" % file2, 3)

    line1 = f1.readline()
    line1 = f1.readline()
    line1 = line1.strip()
    vals = line1.split(",")

    num=0
    for tmp in vals:
	if tmp == conditions['SORT']:
	    break
	num = num+1
    if num == len(vals):
	error_out("there are no col: %s" % conditions['SORT'], 3)

    return num

def compare(c_file, k_file, o_file, file1, file2):
    keys = []
    title = []
    vals = []
    allvals = {}
    sortdata = []

    try:
	    fc = open(c_file)
    except:
	    error_out("Cannot open condition file %s" % c_file, 3)
    try:
	    fo = open(o_file,'w')
    except:
	    error_out("Cannot open output file %s" % o_file, 3)
    try:
	    f1 = open(file1)
    except:
	    error_out("Cannot open file %s" % file1, 3)
    try:
	    f2 = open(file2)
    except:
	    error_out("Cannot open file %s" % file2, 3)

    conditions = read_condition(c_file)
    keys = find_keys(k_file, file1, file2)
    typelen = calc_len(file1, file2)
    title = gen_title(file1, file2)
    num = find_num(file1, file2, conditions)

    allvals = read_allvals(keys, typelen, file1, file2)

    sortdata = data_sort(allvals, typelen, num)

    fo.write(title[0])
    del title[0]
    for tmp in title:
	fo.write(',')
	fo.write(tmp)
    fo.write('\n')

    for val in sortdata:
	fo.write(allvals[val[0]][0])
	del allvals[val[0]][0]
	for tmp in allvals[val[0]]:
	    fo.write(',')
	    fo.write(tmp)
	fo.write('\n')

    fo.write('---condition1:\n')
    cons = conditions['CONDITION'].split(',')
    for con in cons:
	f1 = open(file1)
	for line1 in f1.readlines():
	    line1 = line1.strip()
	    vals = line1.split(",")
	    if con == vals[0]:
		fo.write(vals[0])
		fo.write(',')
		fo.write(vals[1])
		fo.write('\n')
    fo.write('---condition2:\n')
    for con in cons:
	f2 = open(file2)
	for line2 in f2.readlines():
	    line2 = line2.strip()
	    vals = line2.split(",")
	    if con == vals[0]:
		fo.write(vals[0])
		fo.write(',')
		fo.write(vals[1])
		fo.write('\n')

    fo.close()


def main(argv=None):
    import getopt

    condition_file=""
    key_file=""
    output_file=""
    input_file1=""
    input_file2=""
 
    if argv is None:
        argv = sys.argv
    try:
        opts, prog_argv = getopt.getopt(argv[1:], "c:k:o:hv",
                                        ["help", "version",])

    except getopt.error, msg:
        sys.stderr.write("%s: %s\n" % (sys.argv[0], msg))
        sys.stderr.write("Try `%s --help' for more information\n"
                         % sys.argv[0])
        sys.exit(1)

    for opt, val in opts:
        if opt == "-h" or opt == "--help":
            usage(sys.stdout)
            sys.exit(0)

        if opt == "-v" or opt == "--version":
            sys.stdout.write("COMPARER version %d.%d\n" % (MAJOR_VERSION, MINOR_VERSION))
            sys.exit(0)

        if opt == "-c" or opt == "--condition":
            condition_file = val
            continue

        if opt == "-k" or opt == "--key":
            key_file = val
            continue

        if opt == "-o" or opt == "--output":
            output_file = val
            continue

        assert 0, "Should never get here"

    if len(prog_argv) == 2:
        input_file1 = prog_argv[0]
        input_file2 = prog_argv[1]
    else:
        sys.stderr.write("input file?\n")
        sys.stderr.write("Try `%s --help' for more information\n"
                         % sys.argv[0])
        sys.exit(1)

    if (not condition_file) and (not output_file):
	sys.stderr.write("Try `%s --help' for more information\n"
		     % sys.argv[0])
	sys.exit(1)

    #print "%s" % condition_file
    #print "%s" % key_file
    #print "%s" % output_file
    #print "%s" % input_file1
    #print "%s" % input_file2

    compare(condition_file, key_file, output_file, input_file1, input_file2)

if __name__=="__main__":
	main()
