#!/usr/bin/python
#
# KconfigSize tool---distilling individual impact by configuration items
# Copyright (C) 2005-2006 NEC-AS/NEC. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GPLv2, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Created by Lei Wang<wlei@sh.nec-as.nec.com.cn>
# 2006-08-17 Modified by Lei Wang<wlei@sh.nec-as.nec.com.cn>
#

import sys;
import os;
import getopt;

SIZE_RESULT = "---size result";
SIZE_TITLE = "items,effect(raw),effect(compress)";
MEM_RESULT = "---mem result";
MEM_TITLE = "items,effect(used),effect(free),effect(totoal)";
CONDITION = "---conditions";
BASE_CONFIG = "base_config";

conditions = [];

#This function is used to print the help message.
#parameter:
#	null
#return value:
#	null
#
def usage():
	print "indiv -t <type> input output";
	print "type can be \"size\" or \"mem\"";
	return;


# split a string by two with the first key which not in "()".
# parameter:
#       word:           the string you want to split.
#       key:            the key charactor to split with.
# return :
#       a list of the sub string.
def split(word, key):
	uneffect = 0;
	data = [];
	if word[len(word)-1] == '\n' :
		word = word[0:len(word)-1];
	start = 0;
	for i in range(0, len(word)) :
		if i == len(word)-1 :
			data.append(word[start:]);
			break;
		if word[i] == key and uneffect == 0 :
			data.append(word[start:i]);
			start = i+1;
			if start == len(word) :
				data.append("");
				break;
		elif word[i] == '[' :
			uneffect = uneffect + 1;
		elif word[i] == ']' :
			uneffect = uneffect - 1;
	return data;


#This function is used to process the input binder file.
#Every test result item is minused by the base item.
#parameter:
#	input:		the input file name.
#	type:		the input file type. Maybe "size" or "mem".
#return value:
#	a list of processed data.
#
def process(input, type):
	global conditions;

	#check the input file
	if not os.path.isfile(input) :
		print "no such file named %s." % input;
		return [];

	try :
		ifile = open(input, 'r');
	except :
		print "some thing wrong with the file %s" % input;
		return [];
	
	if type == "size" :
		inum = 5;
	else :
		inum = 6;

	lines = ifile.readlines();
	data = [];
	for i in range(0, len(lines)) :
		line = lines[i].strip();
		if line == CONDITION :
			for j in range(i+1, len(lines)) :
				conditions.append(lines[j].strip());
			break;	
		
		items = split(line, ',');
		
		if len(items) < inum or not items[0].isdigit() :
			if items[0].isdigit() :
				if type == "size" :
					if items[0] == "0" :
						print "No base data detected, Indiv process abort! \
						Please check data and retry";
						sys.exit(1);
					else :
						data.append([items[1].strip(), "NONE", "NONE"]);
				else :
					if items[0] == "0" :
						print "No base data detected, Indiv process abort! \
						Please check data and retry";
						sys.exit(1);
					else :
						data.append([items[1].strip(), "NONE", "NONE", "NONE"]);
			continue;

		for j in range(0, len(items)) :
			items[j] = items[j].strip();
			
		if items[0] == "0" :
			if type == "size" :
				data.append(["BASE", items[3], items[4]]);
			else :
				data.append(["BASE", items[3], items[4], items[5]]);
			continue;

		s = [];
		s.append(items[1].strip());

		try :
			for j in range(0, inum-3) :
				s.append(str(int(data[0][j+1]) - int(items[j+3])));
		except :
			print "some thing wrong with the file %s" % input;
			return []; 

		data.append(s);

	return data;


#This function is used to output the processed data to the output file.
#parameter:
#	data:		the processed data list.
#	output:		the output file name.
#	type:		the output file type. Maybe "size" or "mem".
#return value:
#	1		success.
#	0		failed.
#
def saveResult(data, output, type):
	try :
		ofile = open(output, "w");
	except :
		print "Something with the output file. Make sure that you have the right access.";
		return 0;
	
	if type == "size" :
		ofile.write(SIZE_RESULT);
		ofile.write("\n");
		ofile.write(SIZE_TITLE);
		ofile.write("\n");
	else :
		ofile.write(MEM_RESULT);
		ofile.write("\n");
		ofile.write(MEM_TITLE);
		ofile.write("\n");
		
	for d in data :
		i = 0;
		while i < 100 :
			ofile.write(d[i]);
			i = i + 1;
			if i >= len(d) :
				ofile.write("\n");
				break;
			ofile.write(",");

	ofile.write(CONDITION);
	ofile.write("\n");
	for line in conditions :
		ofile.writelines(line);
		ofile.write("\n");

	return 1;


def main(argv):
	(opts, args) = getopt.getopt(argv[1:], "ht:", ["help","type="]);
	if len(args) != 2 :
		usage();
		sys.exit();
	
	#analyse the input arg
	for (opt, value) in opts :
		if opt in ("-t", "--type") :
			type = value;
		if opt in ("-h", "--help") :
			usage();
			sys.exit();
	
	input = args[0];
	output = args[1];
	type = str(type);
	if type != 'size' and type != 'mem' :
		print "The type is wrong.";
		usage();
		sys.exit();
	
	#process the input file
	data = [];
	data = process(input, type);
	if data == [] :
		usage();
		sys.exit();
	
	#output the content of the processed input file
	result = saveResult(data, output, type);
	if result == 0 :
		usage();
		sys.exit();


if __name__ == '__main__' :
	main(sys.argv);
