/*
 * KconfigSize tool---kernel builder
 * Copyright (C) 2005-2006 NEC-AS/NEC. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GPLv2, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Created by Guohuang Yin <yingh@sh.necas.nec.com.cn>
 * 2006-02-20 Modified by Guohuang Yin <yingh@sh.necas.nec.com.cn>
 */

#include "common.h"

#define PROJECT		"BUILDK"
#define AUTHORS		"GuoHuang Yin"

#define	DEFAULT_ARCH		"i386"
#define	DEFAULT_TARGET		"bzImage"
#define	BUILDK_VERSION		"BUILDK 1.0.0"
#define	EXEC_OK			0

void usage (int status);
void check_options (char **argv);
void print_options (FILE *stream);
void kernel_build (void);
int copy_config (char *line,char **index);
int make_mrproper (void);
int make_image (void);
int copy_image (char *index);
int append_buildresult (FILE *fp,char *index,int code);

const char *program_name;
/* variables corresponding to options */
static char *o_arch;
static char *o_config;
static char *o_outdir;
static char *o_kernel;
static char *o_result;
static char *o_target;
static char *o_cross_compile;
static int o_nr_lines;

static const char *const short_options = "a:c:d:hk:o:t:vx:";
static const struct option long_options[] = {
	{ "arch", 1, 0, 'a' },
	{ "config", 1, 0, 'c' },
	{ "outdir", 1, 0, 'd' },
	{ "kernel", 1, 0, 'k' },
	{ "result", 1, 0, 'o' },
	{ "target", 1, 0, 't' },
	{ "cross-compile", 1, 0, 'x' },
	{ GETOPT_HELP_OPTION_DECL },
	{ GETOPT_VERSION_OPTION_DECL },
	{ 0, 0, 0, 0 }
};

int main (int argc,char *argv[])
{
	int c;

	program_name = strrchr (argv[0], '/');
	if (program_name)
		program_name++;
	else
		program_name = argv[0];

	while (1) {
		c = getopt_long (argc, argv, short_options, long_options, 0);
		if(c == -1)
			break;

		switch (c) {
			case 'a':
				/* arch */
				o_arch = optarg;
				break;
			case 'c':
				/* config */
				o_config = optarg;
				break;
			case 'd':
				/* outdir */
				o_outdir = optarg;
				break;
			case 'k':
				/* kernel */
				o_kernel = optarg;
				break;
			case 'o':
				/* result */
				o_result = optarg;
				break;
			case 't':
				/* target */
				o_target = optarg;
				break;
			case 'x':
				/* cross-compile */
				o_cross_compile = optarg;
				break;

			case_GETOPT_HELP_CHAR

			case_GETOPT_VERSION_CHAR(PROJECT, AUTHORS)

			default:
				 /* The user specified an invalid option
				  * or something unexpected.
				  * */
				usage (1);
				break;
		}
	}

	check_options (argv);
	print_options (stdout);
	kernel_build ();
	return 0;
}

void usage (int status)
{
	if (status != 0)
		fprintf (stderr, "Try '%s --help for more information\n",
				program_name);
	else {
		fprintf (stderr, "Usage:%s [option]\n--build kernel tool\n",
			program_name);
		fprintf (stdout,
		"-a,--arch <string>\n"
		"	specify the building kernel architecture\n"
		"	Default:the ARCH of execution environment is used if no"
		" arch is specified,and i386 is use if no ARCH environment\n"
		"-c,--config <file>\n"
		"	specify .config-list file,including some .config files"
							" with special format\n"
		"	It MUST be specified\n"
		"-d,--outdir <dir>\n"
		"	specify the directory where kernel image is copied\n"
		"	Default:current directory if no dir is specified\n"
		"-h,--help\n"
		"	display this help and exit\n"
		"-k,--kernel <dir>\n"
		"	specify kernel source tree\'s top directory\n"
		"	Default:current directory\n"
		"-o,--result <file>\n"
		"	specify the build_result file\n"
		"	Default:current directory\'s build_result is used if no"
							" file is specified\n"
		"-t,--target <string>\n"
		"	specify make target name of the kernel,bzImage for i386,"
								"uImage for arm\n"
		"	Default:default target(bzImage) if no target is specified\n"
		"-v,--version\n"
		"-x,--cross-compile <string>\n"
		"	specify the cross-compile prefix\n"
		"	example: -x arm-linux- will use arm-linux-gcc\n"
		"	Default:gcc is used if no prefix is specified\n");
	}
	exit (status);
}

/*
 * print current options
 */
void print_options (FILE *stream)
{
	fprintf (stream, "Current configure:\n");
	fprintf (stream, "arch:%s\n", o_arch);
	fprintf (stream, "config:%s\n", o_config);
	fprintf (stream, "outdir:%s\n", o_outdir);
	fprintf (stream, "kernel:%s\n", o_kernel);
	fprintf (stream, "result:%s\n", o_result);
	fprintf (stream, "target:%s\n", o_target);
	fprintf (stream, "cross-compile:%s\n", o_cross_compile);
}

/*
 * check optiions one by one
 */
void check_options (char **argv)
{
	int len;
	/* --config option must be set! */
	if (o_config == NULL) {
		fprintf (stderr, "You Must specify the --config option\n");
		usage (1);
	}
	/* --arch option */
	if (o_arch == NULL) {
		o_arch = DEFAULT_ARCH;
		warn (stderr, "arch is not specified, %s is USED!\n",
				DEFAULT_ARCH);
	}
	/* --target option */
	if (o_target == NULL) {
		o_target = DEFAULT_TARGET;
		warn (stderr, "target is not specified, %s is USED!\n",
				DEFAULT_TARGET);
	}
	/* --outdir option */
	if (o_outdir == NULL)
		o_outdir = (char *)dirname (argv[0]);
	else {
		len = strlen (o_outdir);
		if (o_outdir[len - 1] == '/')
			o_outdir[len - 1] = '\0';
	}
	/* --kernel option */
	if (o_kernel == NULL)
		o_kernel = (char *)dirname (argv[0]);
	else {
		len = strlen (o_kernel);
		if (o_kernel[len - 1] == '/')
			o_kernel[len -1] = '\0';
	}

	/* --result option */
	if (o_result == NULL)
		o_result = "./build_result";
}

/*
 * kernel build process:
 * read configlist file,copy cfgfile ,exec make,judge result,
 * copy image and add result to buildresut file
 */
void kernel_build ()
{
	FILE *f_in, *f_out;
	char *line = NULL;
	size_t len = 0;
	char *config_index = NULL;
	int ret;

	/* open configlist file */
	if(!(f_in = fopen (o_config, "r"))) {
		perror ("fopen");
		exit (1);
	}

	/* open buildresul file */
	if(!(f_out = fopen (o_result, "w+"))) {
		perror ("fopen");
		exit (1);
	}

	if (make_mrproper () < 0) {
		perror ("make_mrproper");
		exit (1);
	}

	/* handle config file one by one! */
	while (1) {
		if (getline (&line, &len, f_in) < 0)
			break;

		o_nr_lines++;
		chomp (line);

		ret = copy_config (line, &config_index);
		if (ret < 0) {
			perror ("copy_config");
			exit (1);
		}
		else if (ret == 1)
			continue;
		
		make_image ();
		ret = copy_image (config_index);
		append_buildresult (f_out, config_index, ret);
	}
	if (line)
		free (line);

	if (ferror (f_in) || ferror (f_out)) {
		perror ("ferror");
		exit (2);
	}

	if (fclose (f_out) == EOF) {
		perror ("fclose");
		exit (2);
	}

	if (fclose (f_in) == EOF) {
		perror ("fclose");
		exit (2);
	}
}

/*
 * copy config file to kernel dir
 */
int copy_config (char *line, char **index)
{
	char *srccfg;
	char *destcfg = NULL;
	char *ptr;
	size_t len;
	int ret;

	ptr = strchr (line, ',');
	/* this config file is not handled if format is wrong! */
	if (!ptr) {
		warn (stderr, "Invalid format: File %s, Line %d\n",
				o_config, o_nr_lines);
		return 1;
	}
	else
		*ptr++ = 0;

	*index = trim (line);
	srccfg = trim (ptr);

	if ((**index == '\0') || (*srccfg == '\0')) {
		warn (stderr, "Invalid format: File %s, Line %d\n",
				o_config, o_nr_lines);
		return 1;
	}

	len = 100;
	stradd (&destcfg, o_kernel, &len);
	stradd (&destcfg, "/.config", &len);

	ret = execl_command ("cp", srccfg, destcfg, NULL);

	free (destcfg);

	return ret;
}

int make_mrproper (void)
{
	return execl_command ("make", "-C", o_kernel, "mrproper", NULL);
}

/*
 * kernel build according to one config file
 */
int make_image (void)
{
	char *p_arch = NULL;
	char *p_cross_compile = NULL;
	int ret;
	size_t len = 100;

	stradd (&p_arch, "ARCH=", &len);
	stradd (&p_arch, o_arch, &len);

	if (o_cross_compile) {
		len = 100;
		stradd (&p_cross_compile, "CROSS_COMPILE=", &len);
		stradd (&p_cross_compile, o_cross_compile, &len);
		ret = execl_command ("make", "-C", o_kernel, o_target,\
					p_arch, p_cross_compile, NULL);
		free (p_cross_compile);
	}
	else
		ret = execl_command ("make", "-C", o_kernel, o_target,\
							p_arch, NULL);
	free (p_arch);

	return	ret;
}

/*
 * copy image to outdir including vmlinux and o_target
 */
int copy_image (char *index)
{
	char *src = NULL;
	char *dest = NULL;
	int ret = 0;
	int i;
	size_t len = 100;

	stradd (&src, o_kernel, &len);
	stradd (&src, "/vmlinux", &len);

	len = 100;
	stradd (&dest, o_outdir, &len);
	stradd (&dest, "/", &len);
	stradd (&dest, index, &len);
	stradd (&dest, ".vmlinux", &len);

	i = execl_command ("mv", src, dest, NULL);
	if ( i == EXEC_OK)
		ret = 1;

	free (src);
	free (dest);

	src = NULL;
	dest = NULL;

	len = 100;
	stradd (&src, o_kernel, &len);
	stradd (&src, "/arch/", &len);
	stradd (&src, o_arch, &len);
	stradd (&src, "/boot/", &len);
	stradd (&src, o_target, &len);

	len = 100;
	stradd (&dest, o_outdir, &len);
	stradd (&dest, "/", &len);
	stradd (&dest, index, &len);
	stradd (&dest, ".", &len);
	stradd (&dest, o_target, &len);

	i = execl_command ("mv", src, dest, NULL);
	if (i == EXEC_OK)
		ret |= 2;

	free (src);
	free (dest);
	return ret;
}

/*
 * append result according build_result file format
 */
int append_buildresult (FILE *fp, char *index, int code)
{
	char *buf = NULL;
	size_t len = 250;

	stradd (&buf, index, &len);
	stradd (&buf, ",", &len);
	if (code & 1) {
		stradd (&buf, o_outdir, &len);
		stradd (&buf, "/", &len);
		stradd (&buf, index, &len);
		stradd (&buf, ".vmlinux", &len);
	}
	else
		stradd (&buf, "NONE", &len);

	stradd (&buf, ",", &len);
	if (code & 2) {
		stradd (&buf, o_outdir, &len);
		stradd (&buf, "/", &len);
		stradd (&buf, index, &len);
		stradd (&buf, ".", &len);
		stradd (&buf, o_target, &len);
	}
	else
		stradd (&buf, "NONE", &len);

	stradd (&buf, "\n", &len);	
	fputs (buf, fp);
	fflush (fp);
	free (buf);

	return 0;
}
