/*
 * KconfigSize tool---Interfaces for executing external command
 * Copyright (C) 2005-2006 NEC-AS/NEC. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GPLv2, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Created by YongHua Zheng <zhengyh@sh.necas.nec.com.cn>
 * 2006-02-20 Modified by YongHua Zheng <zhengyh@sh.necas.nec.com.cn>
 */

#include "common.h"

int wait_child(pid_t child)
{
	int status;
	pid_t pid;

	do {
#if defined (HAVE_WAIT4)
		if (pid = wait4(child, &status, 0, NULL) < 0
				&& errno == EINTR) {
#else
		if (pid = waitpid(child, &status, 0) < 0 &&
				errno == EINTR) {
#endif
			sleep (1);
			continue;
		}
		else break;
	} while (!WIFEXITED (status));

	if (pid < 0)
		return -1;

	if (WEXITSTATUS (status) != 0)
		return -1;

	return 0;
}

int execl_command (const char *file, ...)
{
	pid_t child;
	va_list argv, tmp_argv;
	int n_ac;
	const char **av = NULL;
	const char *p;

	if (file == NULL) {
		errno = EINVAL;
		return -1;
	}

	va_start (argv, file);

	/* Count the parameters */
	va_copy (tmp_argv, argv);
	n_ac = 0;
	while (va_arg (tmp_argv, const char *) != NULL)
		n_ac++;

	/* We need two more for argv[0] and NULL terminator */
	n_ac += 2;
	av = (const char **)malloc (n_ac * sizeof (char *));
	if (!av)
		return -1;

	n_ac = 0;
	av[n_ac++] = file;
	while ((p = va_arg (argv, const char *)) != NULL)
		av[n_ac++] = p;
	va_end (argv);
	av[n_ac] = NULL;

	switch (child = fork ()) {
		case 0:
			/* Child process */
			execvp (file, av);
			exit (1);
		case -1:
			free (av);
			return -1;
		default:
			break;
	}

	/* Child will never reach here */
	free (av);

	return wait_child (child);
}

int execv_command (const char *file, const char *av[])
{
	pid_t child;

	if (file == NULL) {
		errno = EINVAL;
		return -1;
	}

	switch (child = fork ()) {
		case 0:
			execvp (file, av);
			exit (1);
		case -1:
			return -1;
		default:
			break;
	}

	return wait_child (child);
}
