/*
 * KconfigSize tool---Interfaces for executing external command
 * Copyright (C) 2005-2006 NEC-AS/NEC. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GPLv2, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Created by YongHua Zheng <zhengyh@sh.necas.nec.com.cn>
 * 2006-02-20 Modified by YongHua Zheng <zhengyh@sh.necas.nec.com.cn>
 */

#include "common.c"

#if !defined (MIN_CHUNK)
#define MIN_CHUNK 64
#endif

int readline (int fd, void **bufptr, size_t *n)
{
	int avail;
	char *read_pos;
	int min_chunk = (*n > MIN_CHUNK)? *n : MIN_CHUNK;
	int ret = 0;
	size_t res;
	char c;
	char *tmptr;

	/* initialize read buffers */
	if (!*bufptr) {
		*n = min_chunk + 1;
		*bufptr = malloc (*n);
		if (!*bufptr)
			return -1;
	}

	avail = *n;
	read_pos = *bufptr;
	while (1) {
		res = full_read (fd, (void *)&c, 1);
		if (res > 0) {
			/* ignore any character we dont want */
			if (strchr ("\r", c))
				continue;

			ret++;
			if (avail < 2) {
				if (*n > min_chunk)
					*n *= 2;
				else
					*n += min_chunk;

				avail = *n + *bufptr - (int)read_pos;
				tmptr = realloc (*bufptr, *n);
				/* make sure it succeded */
				if (!tmptr) {
					free (*bufptr);
					*bufptr = NULL;
					return -1;
				}
				/* then set it */
				*bufptr = tmptr;

				read_pos = *n - avail + *bufptr;
			}
		
			*read_pos++ = c;
			avail--;

			if (c == '\n')
				break;
		}
		else break;
	}
	*read_pos = '\0';

	if (res == (size_t) -1)
		return -1;

	return ret;
}
