/*
 * KconfigSize tool---common functions shared between size1.c and size2.c
 * Copyright (C) 2005-2006 NEC-AS/NEC. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GPLv2, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Created by Lv Zheng <zhengl@sh.necas.nec.com.cn>
 * 2006-02-20 Modified by  Fan Wu<wuf@sh.necas.nec.com.cn>
 * 2006-08-22 Modified by  Guohuang Yin<yingh@sh.nec-as.nec.com.cn>
 *             some funcitions shared between size1.c and size2.c
 */

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <libgen.h>
#include <unistd.h>
#include <time.h>
#include <sys/stat.h>
#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <fcntl.h>
#include <getopt.h>

#define LKC_DIRECT_LINK
#include "lkc.h"

#include "size-common.h"

signed char line[128];
int indent = 1;
int conf_cnt;
int focus_no = 0;
char *result_path;
char absolute_result_path[256];
static struct menu *rootEntry;

void size_conf_value(struct symbol *sym, const char *def)
{
	enum symbol_type type = sym_get_type(sym);
	tristate val;

	line[0] = '\n';
	line[1] = 0;

	if (sym_has_value(sym))
		return;
	switch (type) {
	case S_INT:
	case S_HEX:
	case S_STRING:
		break;
	case S_TRISTATE:
	case S_BOOLEAN:
		if (sym_tristate_within_range(sym, no)) {
			line[0] = 'n';
			line[1] = '\n';
			line[2] = 0;
		}
		break;
	default:
		;
	}

	sym->flags &= ~SYMBOL_NEW;
}

int size_conf_string(struct menu *menu)
{
	struct symbol *sym = menu->sym;
	const char *def, *help;

	while (1) {
		def = size_get_string_value(sym);
		size_conf_value(sym, def);
		switch (line[0]) {
		case '\n':
			break;
		case '?':
		default:
			line[strlen(line)-1] = 0;
			def = line;
		}
		/* TODO: here sym?size? */
		if (def && size_set_string_value(sym, def))
			return TRUE;
	}
}

int size_conf_sym(struct menu *menu)
{
	struct symbol *sym = menu->sym;
	int type;
	tristate oldval, newval;
	const char *help;

	while (1) {
		type = sym_get_type(sym);
		oldval = size_get_tristate_value(sym);
		size_conf_value(sym, size_get_string_value(sym));
		strip(line);

		switch (line[0]) {
		case 'n':
		case 'N':
			newval = no;
			if (!line[1] || !strcmp(&line[1], "o"))
				break;
			continue;
		case 'm':
		case 'M':
			newval = mod;
			if (!line[1])
				break;
			continue;
		case 'y':
		case 'Y':
			newval = yes;
			if (!line[1] || !strcmp(&line[1], "es"))
				break;
			continue;
		case 0:
			newval = oldval;
			break;
		case '?':
			goto help;
		default:
			continue;
		}
		if (size_set_tristate_value(sym, newval))
			return TRUE;
help:
		;
	}
}

void size_conf_menu(struct menu *menu)
{
	struct symbol *sym;
	struct property *prop;
	struct menu *child;

	if (!menu_is_visible(menu))
		return;

	sym = menu->sym;
	prop = menu->prompt;
	if (prop) {
		const char *prompt;

		switch (prop->type) {
		case P_MENU:
			if (rootEntry != menu) {
				size_conf_check(menu);
				return;
			}
		case P_COMMENT:
			prompt = menu_get_prompt(menu);
		default:
			;
		}
	}

	if (!sym)
		goto conf_childs;

	if (sym_is_choice(sym)) {
		size_conf_choice(menu);
		if (sym->curr.tri != mod)
			return;
		goto conf_childs;
	}

	switch (sym->type) {
	case S_INT:
	case S_HEX:
	case S_STRING:
		size_conf_string(menu);
		break;
	default:
		size_conf_sym(menu);
		break;
	}

conf_childs:
	if (sym)
		indent += 2;
	for (child = menu->list; child; child = child->next)
		size_conf_menu(child);
	if (sym)
		indent -= 2;
}

void size_conf_check(struct menu *menu)
{
	struct symbol *sym;
	struct menu *child;

	if (!menu_is_visible(menu))
		return;

	sym = menu->sym;
	if (sym) {
		if (sym_is_changable(sym) && !sym_has_value(sym)) {
			if (!conf_cnt++)
				;
			rootEntry = menu_get_parent_menu(menu);
			size_conf_menu(rootEntry);
		}
		if (sym_is_choice(sym) && sym_get_tristate_value(sym) != mod)
			return;
	}

	for (child = menu->list; child; child = child->next)
		size_conf_check(child);
}

inline bool sym_is_const(struct symbol *sym)
{
	if (!sym)
		return TRUE;
	return sym->flags & SYMBOL_CONST;
}

inline tristate reverse_value(tristate val)
{
	return val == yes || val == mod ? no : yes;
}

inline enum expr_type reverse_expr(enum expr_type type)
{
	switch (type) {
	case E_EQUAL:
		return E_UNEQUAL;
	case E_UNEQUAL:
		return E_UNEQUAL;
	case E_AND:
		return E_OR;
	case E_OR:
		return E_AND;
	default:
		return type;
	}
}

tristate size_get_tristate_value(struct symbol *sym)
{
	if(sym->visible  == yes)
		return sym->user.tri;
	return sym->curr.tri;
}

const char *size_get_string_value(struct symbol *sym)
{
	tristate val;

	if (sym_is_const(sym))
		return sym->curr.val;

	switch (sym->type) {
	case S_BOOLEAN:
	case S_TRISTATE:
		val = size_get_tristate_value(sym);
		switch (val) {
		case no:
			return "n";
		case mod:
			return "m";
		case yes:
			return "y";
		}
		break;
	default:
		;
	}
	return sym->user.val ? (const char *)sym->user.val : sym_get_string_value(sym);
}

bool size_set_tristate_value(struct symbol *sym, tristate val)
{
	tristate oldval = size_get_tristate_value(sym);

	if (sym->flags & SYMBOL_NEW) {
		sym->flags &= ~SYMBOL_NEW;
		sym_set_changed(sym);
	}
	if (sym_is_choice_value(sym) && val == yes) {
		struct symbol *cs = prop_get_symbol(sym_get_choice_prop(sym));

		cs->user.val = sym;
		cs->flags &= ~SYMBOL_NEW;
	}

	sym->user.tri = val;
	sym->curr.tri = val;
	if (oldval != val) {
		sym_clear_all_valid();
		if (sym == modules_sym)
			sym_set_all_changed();
	}

	return true;
}

bool size_set_string_value(struct symbol *sym, const char *newval)
{
	const char *oldval;
	char *val;
	int size;

	switch (sym->type) {
	case S_BOOLEAN:
	case S_TRISTATE:
		switch (newval[0]) {
		case 'y': case 'Y':
			return size_set_tristate_value(sym, yes);
		case 'm': case 'M':
			return size_set_tristate_value(sym, mod);
		case 'n': case 'N':
			return size_set_tristate_value(sym, no);
		}
		return false;
	default:
		;
	}

	if (sym->flags & SYMBOL_NEW) {
		sym->flags &= ~SYMBOL_NEW;
		sym_set_changed(sym);
	}

	oldval = sym->user.val;
	size = strlen(newval) + 1;
	if (sym->type == S_HEX && (newval[0] != '0' || (newval[1] != 'x' && newval[1] != 'X'))) {
		size += 2;
		sym->user.val = val = malloc(size);
		*val++ = '0';
		*val++ = 'x';
	} else if (!oldval || strcmp(oldval, newval))
		sym->user.val = val = malloc(size);
	else
		return true;

	strcpy(val, newval);
	free((void *)oldval);
	sym_clear_all_valid();

	return true;
}

tristate size_calc_expr(struct expr *e)
{
	tristate val1, val2;
	const char *str1, *str2;

	if (!e)
		return yes;

	switch (e->type) {
	case E_SYMBOL:
		sym_calc_value(e->left.sym);
		return size_get_tristate_value(e->left.sym);
	case E_AND:
		val1 = size_calc_expr(e->left.expr);
		val2 = size_calc_expr(e->right.expr);
		return E_AND(val1, val2);
	case E_OR:
		val1 = size_calc_expr(e->left.expr);
		val2 = size_calc_expr(e->right.expr);
		return E_OR(val1, val2);
	case E_NOT:
		val1 = size_calc_expr(e->left.expr);
		return E_NOT(val1);
	case E_EQUAL:
		sym_calc_value(e->left.sym);
		sym_calc_value(e->right.sym);
		str1 = size_get_string_value(e->left.sym);
		str2 = size_get_string_value(e->right.sym);
		return !strcmp(str1, str2) ? yes : no;
	case E_UNEQUAL:
		sym_calc_value(e->left.sym);
		sym_calc_value(e->right.sym);
		str1 = size_get_string_value(e->left.sym);
		str2 = size_get_string_value(e->right.sym);
		return !strcmp(str1, str2) ? no : yes;
	default:
		printf("size_calc_expr: %d?\n", e->type);
		return no;
	}
}

bool size_set_choice_value(struct symbol *entry, struct symbol *value)
{
	entry->user.val = value;
	return size_set_tristate_value(value, yes);
}

/*
 * Generate .config and copy to output dir
 */
void size_gen_config(void)
{
	char index_path[256];

	if (conf_write(NULL)) {
		fprintf(stderr, _("\n*** Error: during writing configuration of focus_no(%d).\n\n"),focus_no);
	}

	sprintf(index_path, "%s" DIRDELIM "%d.config", result_path, focus_no);
	copy_file(".config", index_path);
}


void size_print_symbol(FILE *out, struct menu *menu, struct menu *parent)
{
	struct symbol *sym;
	struct property *prop;
	char *parent_name = NULL;
	char *child_sym_name = NULL; 
	char *child_prompt_text = NULL; 
	int parent_menu_flag =0;
	int child_menu_flag =0;
	
	if(!menu || !parent) return;

	if (parent->sym && parent->sym->name)
		parent_name = parent->sym->name;
	else if (parent->prompt && parent->prompt->text && parent->prompt->type == P_MENU) {
		parent_name = (char*)parent->prompt->text;
		parent_menu_flag =1;
		}
	else
		parent_name = NULL;
	
	sym = menu->sym;
	if (sym) {
		if (sym->name)
			child_sym_name = sym->name;
		prop = sym->prop;
		while(prop) {
			if(prop->type == P_PROMPT){
				child_prompt_text = (char*)prop->text;
				break;
			}
			prop = prop->next;
		}
	} 
	
	if ((!child_sym_name || !child_prompt_text) && (prop = menu->prompt)\
			&& (prop->text) && (prop->type == P_MENU)) {
		child_prompt_text = (char *)prop->text;
		child_menu_flag = 1;
	} 

	if(!child_sym_name && !child_prompt_text)
		return;
	if(!child_prompt_text)
		child_prompt_text = "null";
	if(!child_sym_name)
		child_sym_name = "null";

	/* print the name of symbol */
	if(parent_name) {
		if(parent_menu_flag) {
			if(child_menu_flag && !strcmp(child_sym_name, "null"))
				fprintf(out, "[MENU %s],[MENU %s]\n", parent_name,\
							child_prompt_text);
			else
				fprintf(out, "[MENU %s],[%s],[%s]\n", parent_name,\
						child_sym_name, child_prompt_text);
		} else { 
			if(child_menu_flag && !strcmp(child_sym_name, "null"))
				fprintf(out, "[%s],[MENU %s]\n", parent_name,\
							child_prompt_text);
			else
				fprintf(out, "[%s],[%s],[%s]\n", parent_name,\
						child_sym_name, child_prompt_text);
		}
	}
}


void size_print_menu(FILE *out)
{
	struct symbol *sym;
	struct menu *menu;
	struct property *prop;
	int len;

	menu = rootmenu.list;
	while (menu) {
		size_print_symbol(out, menu, &rootmenu);
		if (menu->list) {
			size_print_sub_menu(out, menu, menu);
		} 
		menu = menu->next;
	}
}

void size_print_sub_menu(FILE *out, struct menu* menu, struct menu* parent)
{
	struct symbol *sym;
	struct menu *submenu;
	struct property *prop;
	char *child_name;
	char *parent_name;

	if(!menu || !parent) return;
	
	submenu = menu->list;
	while (submenu) {
		size_print_symbol(out, submenu, parent);
		if (submenu->list) {
			size_print_sub_menu(out, submenu, submenu);
		}
		submenu = submenu->next;
	}
}

/*
 * util functions
 *
 */
void strip(signed char *str)
{
	signed char *p = str;
	int l;

	while ((isspace(*p)))
		p++;
	l = strlen(p);
	if (p != str)
		memmove(str, p, l + 1);
	if (!l)
		return;
	p = str + l - 1;
	while ((isspace(*p)))
		*p-- = 0;
}

char* get_current_path(void)
{
	memset(absolute_result_path,0,sizeof(absolute_result_path));
	getcwd(absolute_result_path,256);
	return absolute_result_path;
}

int copy_file(const char *from, const char *to)
{
	char buf[1024];
	int ret = FALSE, src = -1, dst = -1;
	struct stat std, st;
	size_t rcnt = 0, wcnt = 0;
	long errnum;
	
	if (from == NULL || to == NULL)
		goto done;
	
	if (stat(from, &st) == -1)
		goto done;
	if (!S_ISREG(st.st_mode))
		goto done;
	
	src = open(from, O_RDONLY);
	if (src == -1)
		goto done;
	
	if (access(to, 00) == 0) {
		if (stat(to, &std) == -1)
			goto done;
		if (!S_ISREG(std.st_mode))
			goto done;
		
		if (chmod(to, st.st_mode) == -1) {
			if (unlink(to) == -1)
				goto done;
		}
	}
	
	dst = open(to, O_CREAT | O_TRUNC | O_WRONLY, st.st_mode);
	if (dst == -1)
		goto done;
	
	while ((rcnt = read(src, buf, sizeof(buf))) > 0) {
		wcnt = write(dst,buf,rcnt);
		if (wcnt == -1 || wcnt != rcnt) {
			close(dst);
			dst = -1;
			errnum = errno;
			unlink(to);
			errno = errnum;
			goto done;
		}
	}
	
	if (rcnt < 0) {
		close(dst);
		dst = -1;
		errnum = errno;
		unlink(to);
		errno = errnum;
		goto done;
	}
	
	ret = TRUE;
done:
	if (src != -1)
		close(src);
	if (dst != -1)
		close(dst);
	
	return ret;
}

char *combine_path(const char *path, const char *file)
{
	char *p = NULL;
	char *delim;
	
	if ((p = malloc(strlen(path) + strlen(file) + 4)) == NULL)
		perror("malloc");

	strcpy(p, path);
	delim = strrchr(path, DIRDELIMC);
	if (!delim || strcmp(DIRDELIM, delim) != 0)
		strcat(p, DIRDELIM);
	strcat(p, file);
	return p;
}

