/*
 * KconfigSize tool---An interface for reading and writting
 * Copyright (C) 2005-2006 NEC-AS/NEC. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GPLv2, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Created by YongHua Zheng <zhengyh@sh.necas.nec.com.cn>
 * 2006-02-20 Modified by YongHua Zheng <zhengyh@sh.necas.nec.com.cn>
 */

#include "common.h"

#if defined (__GNUC__)
__inline
#endif
static size_t _safe_rw (int fd, const void *buf, size_t count, rw rw_func)
{
	ssize_t result;

	/* POSIX limits COUNT to SSIZE_MAX, but we limit it further, requiring
	 * that COUNT <= INT_MAX, to avoid triggering a bug in Tru64 5.1.
	 * When decreasing COUNT, keep the file pointer block-aligned.
	 * Note that in any case, read(write) may succeed, yet read(write)
	 * fewer than COUNT bytes, so the caller must be prepared to handle
	 * partial results.
	 * */
	if (count > INT_MAX)
		count = INT_MAX & ~8191;

	do {
		result = rw_func (fd, buf, count);
	} while (result < 0 && errno == EINTR);

	return (size_t) result;
}

size_t full_rw (int fd, const void *buf, size_t count, rw rw_func)
{
	size_t total = 0;
	const char *ptr = buf;

	while (count > 0) {
		size_t n = _safe_rw (fd, ptr, count, rw_func);
		if (n == SAFE_RW_ERROR)
			return FULL_RW_ERROR;

		if (n == 0)
			break;

		total += n;
		ptr += n;
		count -= n;
	}

	return total;
}
