/*
 * KconfigSize tool---Serial line handling : core part
 * Copyright (C) 2005-2006 NEC-AS/NEC. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GPLv2, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Created by Lv Zheng <zhengl@sh.necas.nec.com.cn>
 * 2006-02-20 Modified by  Mingwei Wang<wangmw@sh.necas.nec.com.cn>
 * 2006-08-18 Modified by  IKEDA, Munehiro <m-ikeda@ds.jp.nec.com>
 */



#include "common.h"
#include "serial_core.h"

#define POSIX_TERMIOS

/* stolen from minicom */
/* Set RTS line. Sometimes dropped. Linux specific? */
static void set_rts(fd)
int fd;
{
#ifdef USE_SOCKET
  if (portfd_is_socket)
    return;
#endif
#if defined(TIOCM_RTS) && defined(TIOCMODG)
  {
    int mcs=0;

    ioctl(fd, TIOCMODG, &mcs);
    mcs |= TIOCM_RTS;
    ioctl(fd, TIOCMODS, &mcs);
  }
#endif
#ifdef _COHERENT
  ioctl(fd, TIOCSRTS, 0);
#endif
}

/* set hardware flow control. */
static void set_hwflow(int fd, int on)
{
	struct termios tty;
	
	tcgetattr(fd, &tty);
	if (on)
		tty.c_cflag |= CRTSCTS;
	else
		tty.c_cflag &= ~CRTSCTS;
	tcsetattr(fd, TCSANOW, &tty);
}


/* open device and return file descriptor */
static int open_device(const char *file)
{
	int fd;

	fd = open(file, O_RDWR | O_NOCTTY | O_NDELAY, S_IREAD | S_IWRITE);
	if (fd < 0) {
		perror("open serial device");
		return fd;
	}

	fcntl(fd, F_SETFL, 0);
	return fd;
}


/* set serial line configuration */
void set_params(int fd, char *baudr, char *par, char *bits,
		char *stopb, int hwf, int swf)
{
	int spd = -1;
	int newbaud;
	int bit = bits[0];
	
	struct termios tty;
	
	tcgetattr(fd, &tty);
	
	/* we generate mark and space parity ourself. */
	if (bit == '7' && (par[0] == 'M' || par[0] == 'S'))
		bit = '8';
	
	/* check if 'baudr' is really a number */
	if ((newbaud = (atol(baudr) / 100)) == 0 && baudr[0] != '0') newbaud = -1;
	
	switch (newbaud) {
	case 0:
#ifdef B0
			spd = B0;	break;
#else
			spd = 0;	break;
#endif
	case 3:		spd = B300;	break;
	case 6:		spd = B600;	break;
	case 12:	spd = B1200;	break;
	case 24:	spd = B2400;	break;
	case 48:	spd = B4800;	break;
	case 96:	spd = B9600;	break;
#ifdef B19200
	case 192:	spd = B19200;	break;
#else /* B19200 */
#  ifdef EXTA
	case 192:	spd = EXTA;	break;
#   else /* EXTA */
	case 192:	spd = B9600;	break;
#   endif /* EXTA */
#endif	 /* B19200 */
#ifdef B38400
	case 384:	spd = B38400;	break;
#else /* B38400 */
#  ifdef EXTB
	case 384:	spd = EXTB;	break;
#   else /* EXTB */
	case 384:	spd = B9600;	break;
#   endif /* EXTB */
#endif	 /* B38400 */
#ifdef B57600
	case 576:	spd = B57600;	break;
#endif
#ifdef B115200
	case 1152:	spd = B115200;	break;
#endif
#ifdef B230400
	case 2304:	spd = B230400;	break;
#endif
	}
	
#if defined (_BSD43) && !defined (POSIX_TERMIOS)
	if (spd != -1) tty.sg_ispeed = tty.sg_ospeed = spd;
	/* number of bits is ignored */
	
	tty.sg_flags = RAW | TANDEM;
	if (par[0] == 'E')
		tty.sg_flags |= EVENP;
	else if (par[0] == 'O')
		tty.sg_flags |= ODDP;
	else
		tty.sg_flags |= PASS8 | ANYP;
	
	ioctl(fd, TIOCSETP, &tty);
	
#  ifdef TIOCSDTR
	/* FIXME: huh? - MvS */
	ioctl(fd, TIOCSDTR, 0);
#  endif
#endif /* _BSD43 && !POSIX_TERMIOS */
	
#if defined (_V7) && !defined (POSIX_TERMIOS)
	if (spd != -1) tty.sg_ispeed = tty.sg_ospeed = spd;
	tty.sg_flags = RAW;
	if (par[0] == 'E')
		tty.sg_flags |= EVENP;
	else if (par[0] == 'O')
		tty.sg_flags |= ODDP;
	
	ioctl(fd, TIOCSETP, &tty);
#endif /* _V7 && !POSIX */
	
#ifdef POSIX_TERMIOS
	
	if (spd != -1) {
		cfsetospeed(&tty, (speed_t)spd);
		cfsetispeed(&tty, (speed_t)spd);
	}
	
	switch (bit) {
	case '5':
		tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS5;
		break;
	case '6':
		tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS6;
		break;
	case '7':
		tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS7;
		break;
	case '8':
	default:
		tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS8;
		break;
	}		
	/* set into raw, no echo mode */
	tty.c_iflag =  IGNBRK;
	tty.c_lflag = 0;
	tty.c_oflag = 0;
	tty.c_cflag |= CLOCAL | CREAD;
#ifdef _DCDFLOW
	tty.c_cflag &= ~CRTSCTS;
#endif
	tty.c_cc[VMIN] = 1;
	tty.c_cc[VTIME] = 5;
	
	if (swf)
		tty.c_iflag |= IXON | IXOFF;
	else
		tty.c_iflag &= ~(IXON|IXOFF|IXANY);
	
	tty.c_cflag &= ~(PARENB | PARODD);
	if (par[0] == 'E')
		tty.c_cflag |= PARENB;
	else if (par[0] == 'O')
		tty.c_cflag |= (PARENB | PARODD);
	
	if (stopb[0] == '2')
		tty.c_cflag |= CSTOPB;
	else
		tty.c_cflag &= ~CSTOPB;
	
	tcsetattr(fd, TCSANOW, &tty);
	
	set_rts(fd);
#endif /* POSIX_TERMIOS */
	
#ifndef _DCDFLOW
	set_hwflow(fd, hwf);
#endif
}


/* setup serial port */
int setup_serial(const char *conf)
{
	FILE *fp;
	int fd_serial;
	char *buf = NULL;
	size_t buf_sz = 0;
	char *key, *val;
	char *baudr = NULL, *par = NULL, *bits = NULL, *stopb = NULL;
	int hwf, swf;
	
	fp = fopen(conf, "r");
	if (!fp) {
		perror(conf);
		return -1;
	}
	
	while ( !feof(fp) ) {
                getline(&buf, &buf_sz, fp);
                chomp(buf);
		key = buf;
		val = strchr(buf, '=');
		if (!val)
			continue;
		*val++ = '\0';
		trim(key);
		trim(val);

                if (strcmp(key, "device") == 0) {
                        fd_serial = open_device(val);
                        if(fd_serial < 0) {
                        	goto end;
                        }
                        continue;
		}
		if (strcmp(key, "speed") == 0) {
			baudr = strdup(val);
			continue;
		}
		
                if (strcmp(key, "parity") == 0) {
			par = strdup(val);
			continue;
		}
		
		if (strcmp(key, "data") == 0) {
			bits = strdup(val);
			continue;
		}
		
		if (strcmp(key, "stopbits") == 0) {
			stopb = strdup(val);
			continue;
		}
		
		if (strcmp(key, "hwflow")) {
			if (strcmp(val, "On") == 0)
				hwf = 1;
			else
				hwf = 0;
			continue;
		}
		
		if (strcmp(key, "swflow")) {
			if (strcmp(val, "On") == 0)
				swf = 1;
			else
				swf = 0;
			continue;
		}
	}
	
	set_params(fd_serial, baudr, par, bits, stopb, hwf, swf);
	
	end :
	
	fclose(fp);
	free(buf);
	free(baudr);
	free(par);
	free(bits);
	free(stopb);
	
	return fd_serial;
}


