/*
 * KconfigSize tool---size tester
 * Copyright (C) 2005-2006 NEC-AS/NEC. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GPLv2, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Created by YongHua Zheng <zhengyh@sh.necas.nec.com.cn>
 * 2006-02-20 Modified by YongHua Zheng <zhengyh@sh.necas.nec.com.cn>
 */

#include "common.h"

#define PROJECT	"sizetest"
#define AUTHORS "YongHua Zheng"

void usage (int status);
int cal_size ();

static char *g_prog_name;
static char *g_oufile;
static FILE *g_in;
static FILE *g_out;
static int  g_lineno;

void usage (int status)
{
	if (status != 0)
		fprintf (stderr, "Try '%s --help' for more information.\n",
				g_prog_name);
	else {
		printf ("Usage: %s <OPTION> <image list file>\n", g_prog_name);
		fputs ("\
Caculate sizes of kernel images specified in kernel list files\n\
\n\
      -o, --output <result file>    specify result file\n\
      -h, --help                    display help\n\
      -v, --version                 display version\n", stdout);
	}
	exit (status);
}

#if defined (__GNUC__)
__inline
#endif
static int strchk (char *c)
{
	if (c) {
		if (*c == '\0')
			return -1;
		return 0;
	}
	return -1;
}

int cal_size ()
{
	size_t size;
	char *linebuff = NULL;
	char *index = NULL, *vmlinux = NULL, *bzImage = NULL;
	char *ptr, *buff;
	int vmlinux_size, bzImage_size;
	int title_printed = 0;
	int strno;

	if (g_in != stdin) {
		if (fseek (g_in, 0L, SEEK_SET) < 0)
			return -1;
	}

	while (!feof (g_in)) {
		if (getline (&linebuff, &size, g_in) < 0)
			break;

		strno = 0;
		g_lineno++;
		chomp (linebuff);

		buff = linebuff;
		while (((ptr = strsplit (&buff, ",")) != NULL) && (strno < 3)) {
			switch (strno) {
				case 0:
					index = ptr;
					break;
				case 1:
					vmlinux = ptr;
					break;
				case 2:
					bzImage = ptr;
					break;
				default:
					assert (0);
			}
			strno++;
		}

		if (strno < 3) {
			warn (NULL, "File %s, line %d: Invalid file format\n",
					g_oufile? g_oufile : "stdin", g_lineno);
			continue;
		}

		if ((strchk (index) == -1)
			|| (strchk (vmlinux) == -1)
			|| (strchk (bzImage) == -1)) {
			warn (NULL, "File %s, line %d: Invalid format\n",
					g_oufile? g_oufile : "stdin", g_lineno);
			continue;
		}

		if (strcmp (vmlinux, "NONE") == 0)
			vmlinux_size = 0;
		else
			vmlinux_size = fsize (vmlinux);
		if (vmlinux_size < 0) {
			warn (NULL, "File %s, Line %d: Failed to get size of vmlinux %s\n",
					g_oufile? g_oufile : "stdin", g_lineno, vmlinux);
			vmlinux_size = 0;
			errno = 0;
		}

		if (strcmp (bzImage, "NONE") == 0)
			bzImage_size = 0;
		else
			bzImage_size = fsize (bzImage);
		if (bzImage_size < 0) {
			warn (NULL, "File %s, line %d: Failed to get size of bzImage %s\n",
					g_oufile? g_oufile : "stdin", g_lineno, bzImage);
			bzImage_size = 0;
			errno = 0;
		}

		if (!title_printed) {
			fprintf (g_out, "%s, %s, %s\n", "index", "raw", "compress");
			title_printed = 1;
		}
		fprintf (g_out, "%s, %d, %d\n", index, vmlinux_size, bzImage_size);
	}

	if (linebuff)
		free (linebuff);

	if (ferror (g_in))
		return -1;

	return 0;
}

static struct option sizetest_options[] = {
	{"output", 1, 0, 'o'},
	{"help", 0, 0, 'h'},
	{"version", 0, 0, 'v'},
	{0,0,0,0}
};

int main (int ac, char *av[])
{
	int option_index;
	int c, ret;

	g_prog_name = strrchr (av[0], '/');
	if (g_prog_name)
		g_prog_name++;
	else
		g_prog_name = av[0];

	g_in = stdin;
	g_out = stdout;

	while (1) {
		c = getopt_long(ac, av, "o:", sizetest_options, &option_index);
		if (c == -1)
			break;

		switch (c) {
			case 'o':
				g_oufile = optarg;
				break;

			case_GETOPT_HELP_CHAR

			case_GETOPT_VERSION_CHAR (PROJECT, AUTHORS)

			default:
				usage (1);
				break;
		}
	}

	if (g_oufile) {
		g_out = fopen (g_oufile, "w");
		if (g_out == NULL) {
			warn (NULL , "Failed to open %s\n", g_oufile);
			exit (1);
		}
	}

	if (optind < ac) {
		g_in = fopen (av[optind], "r");
		if (g_in == NULL) {
			warn (NULL , "Failed to open %s\n", av[optind]);
			exit (1);
		}
	}

	ret = cal_size();
	if (ret < 0) {
		perror ("cal_size");
		exit (1);
	}

	if (g_in != stdin)
		fclose (g_in);
	if (g_out != stdout)
		fclose (g_out);

	return ret;
}

