/*
 * KconfigSize tool---Simple serial line streamer
 * Copyright (C) 2005-2006 NEC-AS/NEC. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GPLv2, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Written by IKEDA, Munehiro <m-ikeda@ds.jp.nec.com>
 */

#include "common.h"
#include "serial_core.h"

static int g_signaled = 0;

static struct option g_long_options[] = {
	{"output", 1, 0, 'o'},
	{"noserial", 0, 0, 'n'},
	{"help", 0, 0, 'h'},
	{0, 0, 0, 0}
};

/* print help message to stderr */
static void usage(const char *cmd)
{
	fprintf(stderr,	"%s : Simple serial line streamer\n", cmd);
	fprintf(stderr,	"Usage: %s [options] CONFIG_FILE\n", cmd);
	fprintf(stderr,	"options:\n");
	fprintf(stderr,	"    -o, --output=<file>  output downstream data to <file>\n"
			"                         instead of stdout\n"
			"                         (for debug use)\n");
	fprintf(stderr,	"    -n, --noserial       use stdin/out instead of serial\n"
			"                         (for debug use)\n");
	fprintf(stderr, "    -h, --help           print this message\n");
}	
	

/* playground between measurement and reset ... dummy implementation */
static void do_after_measure(int fd_ui, int fd_uo, int fd_d)
{
	char *msg = "do_after_measure() is called\n";
	write(fd_d, msg, strlen(msg));
}


/* signal handler for SIGTERM */
static void handl_term(int sig)
{
	g_signaled = 1;
}


int main(int argc, char *argv[])
{
	int fd_upin = -1;	/* data upstream (in) */
	int fd_upout = -1;	/* data upstream (out) */
	int fd_down = 1;	/* data downstream (out only) */
	char *buf = NULL;
	size_t buf_sz = 0;
	char *cmd;
	int c;
	int ret = 0;
	struct sigaction sigact = {
		.sa_handler = handl_term,
		.sa_flags = SA_ONESHOT,
	};
	
	sigemptyset(&sigact.sa_mask);
	cmd = basename(argv[0]);
	
	while (1) {
		c = getopt_long(argc, argv, "o:nh", g_long_options, NULL);
		if (c == -1)
			break;

		switch (c) {
		case 'o' :
			fd_down = open(optarg, O_RDWR|O_CREAT|O_TRUNC,
				S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
			if (fd_down < 0){
				perror(optarg);
				usage(cmd);
				ret = 1;
				goto end;
			}
			break;
		case 'n' :
			fd_upin = 0;	/* stdin */
			fd_upout = 1;	/* stdout */
			break;
		case '?' :
		case 'h' :
		default :
			usage(cmd);
			ret = 1;
			goto end;
		}
	}
	
	if ( (fd_upin == -1) && (optind == argc) ){
		/* use serial, but config file is NOT specified */
		usage(cmd);
		ret = 1;
		goto end;
	}
	
	if (fd_upin == -1) {
		fd_upin = setup_serial(argv[optind]);
		if ( fd_upin < 0 ){
			usage(cmd);
			ret = 1;
			goto end;
		}
		fd_upout = fd_upin;
	}
	
	/* register signal handler for SIGTERM */
	if(sigaction(SIGTERM, &sigact, NULL) < 0) {
		perror("sigaction");
		goto end;
	}
	
	while (1) {
		if (readline(fd_upin, (void**)&buf, &buf_sz) < 0)
			goto end;
		write(fd_down, buf, strlen(buf));
		if (g_signaled) {
			g_signaled = 0;
			do_after_measure(fd_upin, fd_upout, fd_down);
		}
	}	

	end :
	free(buf);
	if (fd_upin < 2)	/* "2" means stderr */
		close(fd_upin);
	if (fd_down < 2)	/* "2" means stderr */
		close(fd_down);
	
	return ret;
}



