/*
 * KconfigSize tool---reset helper via parallel port
 * Copyright (C) 2005-2006 NEC-AS/NEC. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GPLv2, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Created by Ying Huang <huangy@sh.necas.nec.com.cn>
 * 2006-02-20 Modified by Ying Huang <huangy@sh.necas.nec.com.cn>
 */

/* parallel port interface */

unsigned char pp_inb(void);
void pp_outb(unsigned char value);

void pp_init(void);
void pp_done(void);

/* parallel port interface */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/ppdev.h>


/*
   PP_Port+0: Data
   PP_Port+1: Status
   PP_Port+2: Control
*/


/*
 * linux pp access via direct hardware calls
 */

/* address of parallel port */
static unsigned int pp_direct_port;

/* macros for inb and outb */
static __inline unsigned char pp_direct_inb(void)
{
	unsigned char _v;

	__asm__ __volatile__ ("inb %w1,%0":"=a" (_v):"Nd" (pp_direct_port+1));
	return _v;
}

static __inline void pp_direct_outb(unsigned char value)
{
	__asm__ __volatile__ ("outb %b0,%w1": :"a" (value), "Nd" (pp_direct_port));
}

int pp_direct_init(unsigned int port)
{
	pp_direct_port = port;
	if (port == 0)
		port = 0x378;

	/* enable the use of the parallel port */
	if (ioperm(pp_direct_port, 3, 1) != 0) {
		printf ("ERROR: ioperm(0x%x) failed\n", pp_direct_port);
		return(-1);
	}
}

void pp_direct_done(void)
{
}


/*
 * linux pp access via /dev/parport?
 */

/* parallel port device */
static unsigned int pp_ppdev_fd = -1;

unsigned char pp_ppdev_inb(void)
{
	// PP_Port+1=Status
	unsigned char status;

	ioctl(pp_ppdev_fd, PPRSTATUS, &status);

	return(status);
}

void pp_ppdev_outb(unsigned char value)
{
	int i;
	// PP_Port+0=Data
	ioctl(pp_ppdev_fd, PPWDATA, &value);
	for(i = 0; i < 10000; ++i);
}

int pp_ppdev_init(unsigned char *device)
{
	if (!device)
		return(-1);

	if ((pp_ppdev_fd = open(device, O_RDWR)) == -1) {
		printf ("ERROR: open(%s) failed\n", device);
		perror("open");
		return(-1);
	}

	if (ioctl(pp_ppdev_fd, PPCLAIM)) {
		printf ("ERROR: ioctl(PPCLAIM) failed\n");
		perror("ioctl");
		close(pp_ppdev_fd);
		return(-1);
	}
}

void pp_ppdev_done(void)
{
	ioctl(pp_ppdev_fd, PPRELEASE);
	close(pp_ppdev_fd);
}



/*
 * generic parallel port access
 */

unsigned char pp_inb(void)
{
	return pp_ppdev_inb();
//	return pp_direct_inb();
}

void pp_outb(unsigned char value)
{
	pp_ppdev_outb(value);
//	pp_direct_outb(value);
}

void pp_init(void)
{
	pp_ppdev_init("/dev/parport0");
//	pp_direct_init(0);
}

void pp_done(void)
{
	pp_ppdev_done();
//	pp_direct_done();
}

int main(int argc, char *argv[])
{
	unsigned int to_write = 0xff;

	sscanf(argv[1], "%x", &to_write);
	pp_init();
	pp_outb(to_write);
	pp_done();
}
